@@ -2,6 +2,7 @@
 import sys
 import os
 import inspect
+import traceback
 import yaml
 import pycurl
 import json
@@ -9,33 +10,15 @@ import csv
 import logging
 from optparse import OptionParser
 
-# Allow execution from anywhere as long as library is installed
-is_root_folder = False
-try:
-    import binding
-    is_root_folder = True
-except ImportError:
-    pass
-
-if is_root_folder:  # Inside the module
-    from binding import Context
-    import generators
-    from generators import parse_generator
-    from parsing import flatten_dictionaries, lowercase_keys, safe_to_bool, safe_to_json
-    import validators
-    from validators import Failure
-    from tests import Test, DEFAULT_TIMEOUT
-    from benchmarks import Benchmark, AGGREGATES, METRICS, parse_benchmark
-else:  # Importing as library
-    from pyresttest.binding import Context
-    import pyresttest.generators
-    from pyresttest.generators import parse_generator
-    from pyresttest.parsing import flatten_dictionaries, lowercase_keys, safe_to_bool, safe_to_json
-    import pyresttest.validators
-    from pyresttest.validators import Failure
-    from pyresttest.tests import Test, DEFAULT_TIMEOUT
-    from pyresttest.benchmarks import Benchmark, AGGREGATES, METRICS, parse_benchmark
-
+# Pyresttest internals
+from binding import Context
+import generators
+from generators import parse_generator
+from parsing import flatten_dictionaries, lowercase_keys, safe_to_bool, safe_to_json
+import validators
+from validators import Failure
+from tests import Test, DEFAULT_TIMEOUT
+from benchmarks import Benchmark, AGGREGATES, METRICS, parse_benchmark
 """
 Executable class, ties everything together into the framework.
 Module responsibilities:
@@ -46,13 +29,15 @@ Module responsibilities:
 - Perform analysis on benchmark results
 """
 
-
 LOGGING_LEVELS = {'debug': logging.DEBUG,
     'info': logging.INFO,
     'warning': logging.WARNING,
     'error': logging.ERROR,
     'critical': logging.CRITICAL}
 
+logging.basicConfig(format='%(levelname)s:%(message)s')
+logger = logging.getLogger('pyresttest')
+
 class cd:
     """Context manager for changing the current working directory"""
     # http://stackoverflow.com/questions/431684/how-do-i-cd-in-python/13197763#13197763
@@ -176,7 +161,7 @@ def parse_testsets(base_url, test_structure, test_files = set(), working_directo
                 if key == u'import':
                     importfile = node[key] #import another file
                     if importfile not in test_files:
-                        logging.debug("Importing test sets: " + importfile)
+                        logger.debug("Importing test sets: " + importfile)
                         test_files.add(importfile)
                         import_test_structure = read_test_file(importfile)
                         with cd(os.path.dirname(os.path.realpath(importfile))):
@@ -269,9 +254,10 @@ def run_test(mytest, test_config = TestConfig(), context = None):
 
     try:
         curl.perform() #Run the actual call
-    except Exception as e:
+    except Exception, e:
         # Curl exception occurred (network error), do not pass go, do not collect $200
-        result.failures.append(Failure(message="Curl Exception: {0}".format(e), details=str(e)))
+        trace = traceback.format_exc()
+        result.failures.append(Failure(message="Curl Exception: {0}".format(e), details=trace, failure_type=validators.FAILURE_CURL_EXCEPTION))
         result.passed = False
         curl.close()
         return result
@@ -282,7 +268,7 @@ def run_test(mytest, test_config = TestConfig(), context = None):
     response_code = curl.getinfo(pycurl.RESPONSE_CODE)
     result.response_code = response_code
 
-    logging.debug("Initial Test Result, based on expected response code: "+str(response_code in mytest.expected_status))
+    logger.debug("Initial Test Result, based on expected response code: "+str(response_code in mytest.expected_status))
 
     if response_code in mytest.expected_status:
         result.passed = True
@@ -290,7 +276,7 @@ def run_test(mytest, test_config = TestConfig(), context = None):
         # Invalid response code
         result.passed = False
         failure_message = "Invalid HTTP response code: response code {0} not in expected codes [{1}]".format(response_code, mytest.expected_status)
-        result.failures.append(Failure(message=failure_message, details=None))
+        result.failures.append(Failure(message=failure_message, details=None, failure_type=validators.FAILURE_INVALID_RESPONSE))
 
     #print str(test_config.print_bodies) + ',' + str(not result.passed) + ' , ' + str(test_config.print_bodies or not result.passed)
 
@@ -298,29 +284,30 @@ def run_test(mytest, test_config = TestConfig(), context = None):
     if test_config.print_bodies or not result.passed:
         if test_config.interactive:
             print "RESPONSE:"
-        print result.body
+        print result.body.decode("string-escape")
 
     # execute validator on body
     if result.passed is True:
         body = result.body
         if mytest.validators is not None and isinstance(mytest.validators, list):
-            logging.debug("executing this many validators: " + str(len(mytest.validators)))
+            logger.debug("executing this many validators: " + str(len(mytest.validators)))
             failures = result.failures
             for validator in mytest.validators:
                 # TODO add header parsing
                 validate_result = validator.validate(body=body, context=my_context)
                 if not validate_result:
                     result.passed = False
-                if isinstance(validate_result, Failure):
+				# Proxy for checking if it is a Failure object, because of import issues with isinstance there
+                if hasattr(validate_result, 'details'):
                     failures.append(validate_result)
                 # TODO add printing of validation for interactive mode
         else:
-            logging.debug("no validators found")
+            logger.debug("no validators found")
 
         # Only do context updates if test was successful
         mytest.update_context_after(result.body, my_context)
 
-    logging.debug(result)
+    logger.debug(result)
 
     curl.close()
     return result
@@ -362,7 +349,7 @@ def run_benchmark(benchmark, test_config = TestConfig(), context = None):
     curl = pycurl.Curl()
 
     #Benchmark warm-up to allow for caching, JIT compiling, on client
-    logging.info('Warmup: ' + message + ' started')
+    logger.info('Warmup: ' + message + ' started')
     for x in xrange(0, warmup_runs):
         benchmark.update_context_before(my_context)
         templated = benchmark.realize(my_context)
@@ -370,9 +357,9 @@ def run_benchmark(benchmark, test_config = TestConfig(), context = None):
         curl.setopt(pycurl.WRITEFUNCTION, lambda x: None) #Do not store actual response body at all.
         curl.perform()
 
-    logging.info('Warmup: ' + message + ' finished')
+    logger.info('Warmup: ' + message + ' finished')
 
-    logging.info('Benchmark: ' + message + ' starting')
+    logger.info('Benchmark: ' + message + ' starting')
 
     for x in xrange(0, benchmark_runs):  # Run the actual benchmarks
         # Setup benchmark
@@ -395,7 +382,7 @@ def run_benchmark(benchmark, test_config = TestConfig(), context = None):
 
 
     curl.close()
-    logging.info('Benchmark: ' + message + ' ending')
+    logger.info('Benchmark: ' + message + ' ending')
 
     temp_results = dict()
     for i in xrange(0, len(metricnames)):
@@ -485,6 +472,12 @@ def write_benchmark_csv(file_out, benchmark_result, benchmark, test_config = Tes
 # Method to call when writing benchmark file
 OUTPUT_METHODS = {u'csv' : write_benchmark_csv, u'json': write_benchmark_json}
 
+def log_failure(failure, context=None, test_config=TestConfig()):
+    """ Log a failure from a test """
+    logger.error("Test Failure, failure type: {0}, Reason: {1}".format(failure.failure_type, failure.message))
+    if failure.details:
+        logger.error("Validator/Error details:"+str(failure.details))
+
 def run_testsets(testsets):
     """ Execute a set of tests, using given TestSet list input """
     group_results = dict() #results, by group
@@ -524,14 +517,12 @@ def run_testsets(testsets):
 
             if not result.passed: #Print failure, increase failure counts for that test group
                 # Use result test URL to allow for templating
-                logging.error('Test Failed: '+test.name+" URL="+result.test.url+" Group="+test.group+" HTTP Status Code: "+str(result.response_code))
+                logger.error('Test Failed: '+test.name+" URL="+result.test.url+" Group="+test.group+" HTTP Status Code: "+str(result.response_code))
 
                 # Print test failure reasons
                 if result.failures:
                     for failure in result.failures:
-                        logging.error("Test Failure reason: {0}".format(failure))
-                        if failure.details:
-                            logging.error("Validator/Error details:"+str(failure.details))
+                        log_failure(failure, context=context, test_config=myconfig)
 
                 #Increment test failure counts for that group (adding an entry if not present)
                 failures = group_failure_counts[test.group]
@@ -539,7 +530,7 @@ def run_testsets(testsets):
                 group_failure_counts[test.group] = failures
 
             else: #Test passed, print results
-                logging.info('Test Succeeded: '+test.name+" URL="+test.url+" Group="+test.group)
+                logger.info('Test Succeeded: '+test.name+" URL="+test.url+" Group="+test.group)
 
             #Add results for this test group to the resultset
             group_results[test.group].append(result)
@@ -551,19 +542,19 @@ def run_testsets(testsets):
 
         for benchmark in mybenchmarks:  # Run benchmarks, analyze, write
             if not benchmark.metrics:
-                logging.debug('Skipping benchmark, no metrics to collect')
+                logger.debug('Skipping benchmark, no metrics to collect')
                 continue
 
-            logging.info("Benchmark Starting: "+benchmark.name+" Group: "+benchmark.group)
+            logger.info("Benchmark Starting: "+benchmark.name+" Group: "+benchmark.group)
             benchmark_result = run_benchmark(benchmark, myconfig, context=context)
             print benchmark_result
-            logging.info("Benchmark Done: "+benchmark.name+" Group: "+benchmark.group)
+            logger.info("Benchmark Done: "+benchmark.name+" Group: "+benchmark.group)
 
             if benchmark.output_file:  # Write file
-                logging.debug('Writing benchmark to file in format: '+benchmark.output_format)
+                logger.debug('Writing benchmark to file in format: '+benchmark.output_format)
                 write_method = OUTPUT_METHODS[benchmark.output_format]
                 my_file =  open(benchmark.output_file, 'w')  # Overwrites file
-                logging.debug("Benchmark writing to file: " + benchmark.output_file)
+                logger.debug("Benchmark writing to file: " + benchmark.output_file)
                 write_method(my_file, benchmark_result, benchmark, test_config = myconfig)
                 my_file.close()
 
@@ -585,8 +576,14 @@ def run_testsets(testsets):
 
 def register_extensions(modules):
     """ Import the modules and register their respective extensions """
+    if isinstance(modules, basestring):  # Catch supplying just a string arg
+        modules = [modules]
     for ext in modules:
-        module = __import__(ext)
+        # Get the package prefix and final module name
+        segments = ext.split('.')
+        module = segments.pop()
+        package = '.'.join(segments)
+        module = __import__(ext, globals(), locals(), package)  # Necessary to get the root module back
 
         # Extensions are registered by applying a register function to sets of registry name/function pairs inside an object
         extension_applies = {
@@ -597,11 +594,24 @@ def register_extensions(modules):
             'GENERATORS': generators.register_generator
         }
 
+        has_registry = False
         for registry_name, register_function in extension_applies.items():
             if hasattr(module, registry_name):
                 registry = getattr(module, registry_name)
                 for key, val in registry.items():
                     register_function(key, val)
+                if registry:
+                    has_registry = True
+
+        if not has_registry:
+            raise ImportError("Extension to register did not contain any registries: {0}".format(ext))
+
+# AUTOIMPORTS, these should run just before the main method, to ensure everything else is loaded
+try:
+    import jsonschema
+    register_extensions('ext.validator_jsonschema')
+except ImportError, ie:
+    logging.warn("Failed to load jsonschema validator, make sure the jsonschema module is installed if you wish to use schema validators.")
 
 def main(args):
     """
@@ -616,7 +626,7 @@ def main(args):
     """
 
     if 'log' in args and args['log'] is not None:
-        logging.basicConfig(level=LOGGING_LEVELS.get(args['log'].lower(), logging.NOTSET))
+        logger.setLevel(LOGGING_LEVELS.get(args['log'].lower(), logging.NOTSET))
 
     if 'import_extensions' in args and args['import_extensions']:
         extensions = args['import_extensions'].split(';')
@@ -644,8 +654,8 @@ def main(args):
 
     sys.exit(failures)
 
-#Allow import into another module without executing the main method
-if(__name__ == '__main__'):
+def command_line_run(args_in):
+    """ Runs everything needed to execute from the command line, so main method is callable without arg parsing """
     parser = OptionParser(usage="usage: %prog base_url test_filename.yaml [options] ")
     parser.add_option(u"--print-bodies", help="Print all response bodies", action="store", type="string", dest="print_bodies")
     parser.add_option(u"--log", help="Logging level", action="store", type="string")
@@ -654,7 +664,7 @@ if(__name__ == '__main__'):
     parser.add_option(u"--test", help="Test file to use", action="store", type="string")
     parser.add_option(u'--import_extensions', help='Extensions to import, separated by semicolons', action="store", type="string")
 
-    (args, unparsed_args) = parser.parse_args()
+    (args, unparsed_args) = parser.parse_args(args_in)
     args = vars(args)
 
     # Handle url/test as named, or, failing that, positional arguments
@@ -671,4 +681,8 @@ if(__name__ == '__main__'):
             parser.error("wrong number of arguments, need both url and test filename, either as 1st and 2nd parameters or via --url and --test")
 
     args['cwd'] = os.path.realpath(os.path.abspath(os.getcwd()))  # So modules can be loaded from current folder
-    main(args)
\ No newline at end of file
+    main(args)
+
+#Allow import into another module without executing the main method
+if(__name__ == '__main__'):
+    command_line_run(sys.argv[1:])
\ No newline at end of file
