@@ -1,6 +1,7 @@
 import logging
 import json
 import operator
+import traceback
 import string
 import parsing
 import os
@@ -24,6 +25,9 @@ Validators:
 
 """
 
+logger = logging.getLogger('pyresttest.validators')
+
+
 # Binary comparison tests
 COMPARATORS = {
     'count_eq': lambda x,y: safe_length(x) == y,
@@ -63,12 +67,21 @@ def safe_length(var):
         pass
     return output
 
+# Validator Failure Reasons
+FAILURE_INVALID_RESPONSE = 'Invalid HTTP Response Code'
+FAILURE_CURL_EXCEPTION = 'Curl Exception'
+FAILURE_TEST_EXCEPTION = 'Test Execution Exception'
+FAILURE_VALIDATOR_FAILED = 'Validator Failed'
+FAILURE_VALIDATOR_EXCEPTION = 'Validator Exception'
+FAILURE_EXTRACTOR_EXCEPTION = 'Extractor Exception'
+
 class Failure(object):
     """ Encapsulates why and how a validation failed for user consumption
         Message is a short explanation, details is a longer, multiline reason
         Validator is the validator that failed (for config info)
     """
     message = None
+    failure_type = None
     details = None
     validator = None
 
@@ -79,10 +92,11 @@ class Failure(object):
     def __str__(self):
         return self.message
 
-    def __init__(self, message="", details="", validator=None):
+    def __init__(self, message="", details="", failure_type=None, validator=None):
         self.message = message
         self.details = details
         self.validator = validator
+        self.failure_type = failure_type
 
 
 class AbstractExtractor(object):
@@ -114,6 +128,16 @@ class AbstractExtractor(object):
             query = string.Template(query).safe_substitute(context.get_values())
         return query
 
+    def get_readable_config(self, context=None):
+        """ Print a human-readable version of the configuration """
+        query = self.templated_query(context=context)
+        output = 'Extractor Type: {0},  Query: "{1}", Templated?: {2}'.format(self.extractor_type, query, self.is_templated)
+        args_string = None
+        if self.args:
+            args_string = ", Args: " + str(self.args)
+            output = output + args_string
+        return output
+
     @classmethod
     def configure_base(cls, config, extractor_base):
         """ Parse config object and do basic config on an Extractor
@@ -219,11 +243,23 @@ class ComparatorValidator(AbstractValidator):
     expected = None
     isTemplateExpected = False
 
+    def get_readable_config(self, context=None):
+        """ Get a human-readable config string """
+        string_frags = list()
+        string_frags.append("Extractor: "+self.extractor.get_readable_config(context=context))
+        if isinstance(self.expected, AbstractExtractor):
+            string_frags.append("Expected value extractor: "+self.expected.get_readable_config(context=context))
+        elif self.isTemplateExpected:
+            string_frags.append('Expected is templated, raw value: {0}'.format(self.expected))
+        return os.linesep.join(string_frags)
+
+
     def validate(self, body=None, headers=None, context=None):
         try :
             extracted_val = self.extractor.extract(body=body, headers=headers, context=context)
         except Exception as e:
-            return Failure(message="Extractor threw exception", details=e, validator=self)
+            trace = traceback.format_exc()
+            return Failure(message="Extractor threw exception", details=trace, validator=self, failure_type=FAILURE_EXTRACTOR_EXCEPTION)
 
         # Compute expected output, either templating or using expected value
         expected_val = None
@@ -231,7 +267,8 @@ class ComparatorValidator(AbstractValidator):
             try:
                 expected_val = self.expected.extract(body=body, headers=headers, context=context)
             except Exception as e:
-                return Failure(message="Expected value extractor threw exception", details=e, validator=self)
+                trace = traceback.format_exc()
+                return Failure(message="Expected value extractor threw exception", details=trace, validator=self, failure_type=FAILURE_EXTRACTOR_EXCEPTION)
         elif self.isTemplateExpected and context:
             expected_val = string.Template(self.expected).safe_substitute(context.get_values())
         else:
@@ -241,7 +278,8 @@ class ComparatorValidator(AbstractValidator):
         if not comparison:
             failure = Failure(validator=self)
             failure.message = "Comparison failed, evaluating {0}({1}, {2}) returned False".format(self.comparator_name, extracted_val, expected_val)
-            failure.details = self.config
+            failure.details = self.get_readable_config(context=context)
+            failure.failure_type = FAILURE_VALIDATOR_FAILED
             return failure
         else:
             return True
@@ -307,6 +345,10 @@ class ExtractTestValidator(AbstractValidator):
     test_name = None
     config = None
 
+    def get_readable_config(self, context=None):
+        """ Get a human-readable config string """
+        return "Extractor: "+self.extractor.get_readable_config(context=context)
+
     @staticmethod
     def parse(config):
         output = ExtractTestValidator()
@@ -325,14 +367,16 @@ class ExtractTestValidator(AbstractValidator):
         try:
             extracted = self.extractor.extract(body=body, headers=headers, context=context)
         except Exception as e:
-            return Failure(message="Exception thrown while running extraction from body", details=e, validator=self)
+            trace = traceback.format_exc()
+            return Failure(message="Exception thrown while running extraction from body", details=trace, validator=self, failure_type=FAILURE_EXTRACTOR_EXCEPTION)
 
         tested = self.test_fn(extracted)
         if tested:
             return True
         else:
-            failure = Failure(details=self.config, validator=self)
+            failure = Failure(details=self.get_readable_config(context=context), validator=self, failure_type=FAILURE_VALIDATOR_FAILED)
             failure.message = "Extract and test validator failed on test: {0}({1})".format(self.test_name, extracted)
+            # TODO can we do better with details?
             return failure
 
 def parse_extractor(extractor_type, config):
